//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-pdf-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.IO.Compression;
using System.Drawing;
using System.Collections.Generic;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Common;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // This sample shows how to use the SavePdfOptions.UseObjectStreams property
    // to reduce the size of a PDF.
    //
    // Note that as with most optimizations, the size reduction will vary depending
    // on the actual content of the source PDF.
    public class ObjectStreams
    {
        public int CreatePDF(Stream stream)
        {
            // Create a 5 page non-optimal PDF:
            var tmpInput = MakeInputFile();
            var fiInput = new FileInfo(tmpInput);

            // Create a new PDF, load the source PDF into it, and optimize it:
            var tmpOutput = Path.GetTempFileName();
            var tmpDoc = new GcPdfDocument();
            using (var fs = File.OpenRead(tmpInput))
            {
                tmpDoc.Load(fs);
                // By default GcPdfDocument uses CompressionLevel.Fastest when saving a PDF.
                // If PDF size is important, we should use optimal compression:
                tmpDoc.CompressionLevel = CompressionLevel.Optimal;
                // Minimize stream sizes, use object streams:
                tmpDoc.Save(tmpOutput, new SavePdfOptions(SaveMode.Default, PdfStreamHandling.MinimizeSize, UseObjectStreams.Multiple));
            }
            var fiOutput = new FileInfo(tmpOutput);

            // Record the input and output file sizes in the resultant PDF:
            var doc = new GcPdfDocument();
            Common.Util.AddNote(String.Format(
                "Using the UseObjectStreams.Multiple option when saving a PDF will in most cases reduce the resulting file size, " +
                "sometimes significantly. In this case the size of the PDF generated by the 'Large Document' sample decreased " +
                "from {0:N0} to {1:N0} bytes, without any loss in fidelity or PDF opening speed.\n" +
                "Using the UseObjectStreams.Single option yields an even slightly smaller PDF size at the cost of slower opening in PDF viewers.\n" +
                "To reproduce these results locally, download and run this sample, specifying a valid license key " +
                "(otherwise loading is limited to 5 pages, and the size reduction may be too small). " +
                "You may also modify the sample code to keep the temporary " +
                "input and output files, and compare their sizes using a file manager.", fiInput.Length, fiOutput.Length),
                doc.NewPage());
            doc.Save(stream);

            // Clean up:
            File.Delete(tmpInput);
            File.Delete(tmpOutput);

            return doc.Pages.Count;
        }

        static string MakeInputFile()
        {
            // Number of pages to generate:
            const int N = Common.Util.LargeDocumentIterations;
            var start = Common.Util.TimeNow();
            var doc = new GcPdfDocument();
            // Prep a TextLayout to hold/format the text:
            var tl = new TextLayout(72)
            {
                MaxWidth = doc.PageSize.Width,
                MaxHeight = doc.PageSize.Height,
                MarginAll = 72,
                FirstLineIndent = 36,
            };
            tl.DefaultFormat.Font = StandardFonts.Times;
            tl.DefaultFormat.FontSize = 12;
            // Generate the document:
            for (int pageIdx = 0; pageIdx < N; ++pageIdx)
            {
                tl.Append(Common.Util.LoremIpsum(1));
                tl.PerformLayout(true);
                doc.NewPage().Graphics.DrawTextLayout(tl, PointF.Empty);
                tl.Clear();
            }
            // Insert a title page:
            tl.FirstLineIndent = 0;
            var fnt = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "NotoSerif-Regular.ttf"));
            var tf0 = new TextFormat() { Font = fnt, FontSize = 24, FontBold = true };
            tl.Append(string.Format("Large Document\n{0} Pages of Lorem Ipsum\n\n", N), tf0);
            var tf1 = new TextFormat(tf0) { FontSize = 14, FontItalic = true };
            tl.Append(string.Format("Generated on {0} in {1:m\\m\\ s\\s\\ fff\\m\\s}.", Common.Util.TimeNow().ToString("R"), Common.Util.TimeNow() - start), tf1);
            tl.TextAlignment = TextAlignment.Center;
            tl.PerformLayout(true);
            doc.Pages.Insert(0).Graphics.DrawTextLayout(tl, PointF.Empty);

            // Save the resultant PDF in a temp file with UseObjectStreams.None (it is the default):
            var outFn = Path.GetTempFileName();
            doc.Save(outFn, new SavePdfOptions(SaveMode.Default, PdfStreamHandling.Copy, UseObjectStreams.None));
            return outFn;
        }
    }
}
